from builtins import str
from pandas.util.testing import assert_frame_equal
import pytest
import numpy.testing as npt
import pandas as pd
from tests.test_input.constants import *
from datum.quant_file_processing import *

# More Input sets required for all (most) possible input file types

# Input required
@pytest.mark.parametrize('input_local_files', [''])
def test_assert_curvefit_input_files(input_local_files):
    pass


@pytest.mark.parametrize('input_local_files', [''])
def test_assert_curvefit_input_files_exc(input_local_files):
    with pytest.raises(AssertionError) as excinfo:
        assert_curvefit_input_files(input_local_files)
        assert 'Input to the application is invalid' in str(excinfo.value)
    

def test_read_file_exc():
    with pytest.raises(IOError) as err:
        read_file(os.path.join(dir_path, 'input.py'))
        assert erstr(r.value) == 'only csv/xls/xlsx/txt extensions are allowed'


@pytest.mark.parametrize('file_path,exp_df', [(os.path.join(dir_path, 'pool_size.csv'),
                                                pd.read_csv(os.path.join(dir_path, 'pool_size.csv'))),
                                              (os.path.join(dir_path, 'test_text_file.txt'),
                                                pd.read_table(os.path.join(dir_path, 'test_text_file.txt'))) ])
def test_read_file(file_path, exp_df):
    test_df = read_file(file_path)
    assert_frame_equal(test_df, exp_df)


def test_check_mq_column_headers():
    col_names = ['A', 'B', 'C', 'D']
    col_headers = ['A', 'M', 'D', 'C', 'F', 'B']
    check_mq_column_headers(col_headers, col_names)


def test_check_mq_column_header_wildcard():
    test_col_names = ['A', 'B', 'C']
    test_col_header = ['A', 'V', 'C', 'Z']
    with pytest.raises(AssertionError) as e:
        check_mq_column_headers(test_col_header, test_col_names)
    assert str(e.value) == "Required column/s not found, Column: ['B']"


# Need input for the exception flow, where file path is absent in mq dict
#  or cdv raises error
@pytest.mark.parametrize('path,check_dict,exp_output_df', [ (os.path.join(dir_path, 'metadata_mq.csv'), constants.DICT_READ_MULTIQUANT_METADATA, pd.DataFrame(
        {'Component Name': ['L-Cystathionine', 'Pantothenic acid'],
         'Unlabeled Fragment': ['L-Cystathionine', 'Pantothenic acid']}) ),
         (os.path.join(dir_path, 'metadata_std_sample.csv'), constants.DICT_READ_STD_SAMPLE_METADATA, pd.DataFrame(
        {'Sample Name': ['STD_#SVLSCE6_8_1', 'STD_#SVLSCE6_8_2'],
         'Concentration': [3, 10],
         'Unlabeled Fragment': ['Itaconic acid', 'Itaconic acid'],
         'Original Filename': ['STD_#SVLSCE6_8_1', 'STD_#SVLSCE6_8_2'],
         'Unit': ['uM', 'uM']})) ])
def test_read_multiquant_metadata(path, check_dict, exp_output_df):
    df, logs = df_csv_validator(check_dict, path)
    assert isinstance(df, pd.DataFrame)
    assert isinstance(logs, dict)
    assert_frame_equal(df, exp_output_df)


def test_read_std_sample_metadata_wildcard():
    with pytest.raises(AssertionError) as e:
        read_std_sample_metadata(
            os.path.join(dir_path, 'metadata_std_sample_error.csv'))
    assert str(e.value) == "Required column not found, Column containing " \
                              "the word: 'Concentration'"


def test_read_multiquant():
    df = read_multiquant(os.path.join(dir_path, 'test_biosample_mq.xlsx'))
    expected_df = pd.DataFrame({'Sample Name': ['S1', 'S2'],
                                'Area': [164640.276, 5659.200],
                                'Component Name': ['Citrate 191/67',
                                                   'Citrate 192/67'],
                                'Original Filename': ['F1', 'F2']})
    expected_df = expected_df[
        ['Original Filename', 'Sample Name', 'Component Name', 'Area']]
    assert_frame_equal(df, expected_df)


def test__merge_std_df_metadata():
    mqDF = ProcessMqDf(raw_mq_df, metadata_std_sample_df, metadata_mq_df)
    standard_df, biosamples_df = mqDF._split_std_bio()
    merged_std = mqDF._merge_std_df_metadata(standard_df)
    assert np.array_equal(merged_std.columns.values,
                          ['Original Filename', 'Sample Name', 'Component Name',
                           'Mass Info', 'Area', 'Unlabeled Fragment',
                           'Concentration', 'Unit'])


def test__split_std_bio_err():
    mqDF = ProcessMqDf(raw_mq_df, metadata_std_sample_df, metadata_mq_df)
    standard_df, biosamples_df = mqDF._split_std_bio()
    assert all([np.array_equal(standard_df.columns.values,
                               ['Original Filename', 'Sample Name',
                                'Component Name',
                                'Mass Info', 'Area', 'Unlabeled Fragment']),
                np.array_equal(biosamples_df.columns.values,
                               ['index', 'Original Filename', 'Sample Name',
                                'Component Name', 'Mass Info',
                                'Unlabeled Fragment',
                                'Area', 'Concentration'])])


def test__split_std_bio_wildcard():
    mqDF = ProcessMqDf(raw_mq_df_1, metadata_std_sample_df, metadata_mq_df)
    with pytest.raises(ValueError) as e:
        mqDF._split_std_bio()
    assert str(e.value) == 'Std samples not found in Sample Name column'


def test__merge_std_df_metadata_err():
    mqDF = ProcessMqDf(raw_mq_df, metadata_std_sample_df_1, metadata_mq_df)
    standard_df, biosamples_df = mqDF._split_std_bio()
    with pytest.raises(Exception) as e:
        mqDF._merge_std_df_metadata(standard_df)
    assert str(e.value) == 'No common entries for Original Filename,' \
                              'Sample Name or Unlabeled Fragment in ' \
                              'metadata_std_sample and raw files.'


# 1 of 2 assert failing
@pytest.mark.parametrize('processmqdf_object,exp_std_df,exp_bio_std_df', [(ProcessMqDf(raw_mq_df, metadata_std_sample_df, metadata_mq_df),
    ['Original Filename', 'Sample Name',
                                'Component Name',
                                'Mass Info', 'Area', 'Unlabeled Fragment',
                                'Concentration', 'Unit'], ['index', 'Original Filename', 'Sample Name',
                                'Component Name', 'Mass Info',
                                'Unlabeled Fragment',
                                'Area', 'Concentration'])])
def test_get_standard_biosample_data(processmqdf_object, exp_std_df, exp_bio_std_df):
    standard_df, biosamples_df =  processmqdf_object.get_standard_biosample_data()
    assert all([np.array_equal(standard_df.columns.values,
                               exp_std_df),
                np.array_equal(biosamples_df.columns.values,
                               exp_bio_std_df)])


def test_merge_mq_metadata():
    with pytest.raises(AssertionError) as e:
        ProcessMqDf(raw_mq_df, metadata_std_sample_df, metadata_mq_df_1)
    assert str(e.value) == "Component Name must be subset of Unlabeled Fragment"


def test_process_input_data():
    merged_std, biosample, logs = process_input_data(input_local_files)
    assert all([np.array_equal(merged_std.columns.values,
                               ['Original Filename', 'Sample Name',
                                'Component Name', 'Mass Info', 'Area',
                                'Unlabeled Fragment', 'Concentration', 'Unit']),
                np.array_equal(biosample.columns.values,
                               ['index', 'Original Filename',
                                'Sample Name', 'Component Name',
                                'Mass Info', 'Unlabeled Fragment',
                                'Area',
                                'Concentration'])])


def test_get_input_dataframes():
    biosamples, merged_std_df = get_input_dataframes(curve_fit_raw_file)
    assert all([np.array_equal(merged_std_df.columns.values,
                               ['Area', 'Component Name', 'Concentration uM',
                                'Mass Info', 'Original Filename', 'Sample Name',
                                'Unlabeled Fragment', ]),
                np.array_equal(biosamples.columns.values,
                               ['Area', 'Component Name', 'Concentration',
                                'Mass Info',
                                'Original Filename', 'Sample Name',
                                'Unlabeled Fragment',
                                'index'])])


# @pytest.mark.parametrize('keys,obj_check')
# def test_check_for_keys_existence():
#     keys = ["A", "B", "C"]
#     object_to_check = input_local_files
#     with pytest.raises(AssertionError) as e:
#         check_for_keys_existence(keys, object_to_check,
#                                  error_fun=AssertionError,
#                          str(       )="Input to the application is invalid")
#     assert str(e.value) == "Input to the application is invalid"